/**
  * InColumnRecursiveNumberLister
  *
  * elenca i numeri da 1 a n usando un algoritmo ricorsivo
  * I numeri sono incolonnati a dieci per riga e ciascun numero ha 5 caratteri
  * usando, eventialmente, spazi a sinistra.
  * Il numero n e' passato come argomento nella riga di comando.
  * Il risultato e' visualizzato a standard output.
  *
  * @author A. Luchetta
  * @version 11-Nov-2006
*/
public class InColumnRecursiveNumberLister
{
   private static int NUMBERS_PER_LINE = 10;
   private static int DIGITS_PER_NUMBER = 5;

   /**
      elenca ricorsivamente i numeri da 1 a n
      @param n il <numero intero massimo da elencare
      @return la stringa contenente l'elenco dei numeri separati da uno spazio
   */
   public static String recursivelyListNumbers(int n)
   {
      //gestione pre-condizioni
      if (n < 1)
      throw new IllegalArgumentException(n + " valore non ammesso");

      // caratteri per numero
      String nStr = Integer.toString(n);
      while (nStr.length() < DIGITS_PER_NUMBER )
         nStr = " " + nStr;
         
      //caso base
      if ( n <= 1)
         return nStr;

      // numeri per riga
      String delimiter = " ";
      if (((n-1) % NUMBERS_PER_LINE) == 0)
         delimiter = "\n";

      //passo ricorsivo
      String tmp = recursivelyListNumbers(n - 1) + delimiter + nStr;
      return tmp;
   }
   
   /**
      rende la classe eseguibile per provare il metodo recursivelyListNumbers.
      Il numero n e' passato come argomento nella riga di comando.
      Il risultato e' visualizzato a standard output.
   */
   public static void main(String[] args)
   {
      if (args.length < 1)
      {
         System.out.println("uso: $java RecursiveNumberLister string");
         return;
      }

      int k = Integer.parseInt(args[0]);

      String list = recursivelyListNumbers(k);
      System.out.println(list);
   }
}